@@ -11,6 +11,38 @@
 // the License.
 
 (function($) {
+
+  $.toTimeString = function(milliseconds) {
+    var ms, sec, min, h, timeString;
+
+    sec = Math.floor(milliseconds / 1000.0);
+    min = Math.floor(sec / 60.0);
+    sec = (sec % 60.0).toString();
+    if (sec.length < 2) {
+       sec = "0" + sec;
+    }
+
+    h = (Math.floor(min / 60.0)).toString();
+    if (h.length < 2) {
+      h = "0" + h;
+    }
+
+    min = (min % 60.0).toString();
+    if (min.length < 2) {
+      min = "0" + min;
+    }
+
+    timeString = h + ":" + min + ":" + sec;
+
+    ms = (milliseconds % 1000.0).toString();
+    while (ms.length < 3) {
+      ms = "0" + ms;
+    }
+    timeString += "." + ms;
+
+    return timeString;
+  };
+
   $.futon = $.futon || {};
   $.extend($.futon, {
 
@@ -190,26 +222,34 @@
       }
 
       this.databaseSecurity = function() {
+        function namesAndRoles(r, key) {
+          var names = [];
+          var roles = [];
+          if (r && typeof r[key + "s"] === "object") {
+            if ($.isArray(r[key + "s"]["names"])) {
+              names = r[key + "s"]["names"];
+            }
+            if ($.isArray(r[key + "s"]["roles"])) {
+              roles = r[key + "s"]["roles"];
+            }
+          }
+          return {names : names, roles: roles};
+        };
+
         $.showDialog("dialog/_database_security.html", {
           load : function(d) {
             db.getDbProperty("_security", {
               success: function(r) {
-                ["admin", "reader"].forEach(function(key) {
-                  var names = [];
-                  var roles = [];
-
-                  if (r && typeof r[key + "s"] === "object") {
-                    if ($.isArray(r[key + "s"]["names"])) {
-                      names = r[key + "s"]["names"];
-                    }
-                    if ($.isArray(r[key + "s"]["roles"])) {
-                      roles = r[key + "s"]["roles"];
-                    }
-                  }
-
-                  $("input[name=" + key + "_names]", d).val(JSON.stringify(names));
-                  $("input[name=" + key + "_roles]", d).val(JSON.stringify(roles));
-                });
+                var admins = namesAndRoles(r, "admin")
+                  , members = namesAndRoles(r, "member");
+                if (members.names.length + members.roles.length == 0) {
+                  // backwards compatibility with readers for 1.x
+                  members = namesAndRoles(r, "reader");
+                }
+                $("input[name=admin_names]", d).val(JSON.stringify(admins.names));
+                $("input[name=admin_roles]", d).val(JSON.stringify(admins.roles));
+                $("input[name=member_names]", d).val(JSON.stringify(members.names));
+                $("input[name=member_roles]", d).val(JSON.stringify(members.roles));
               }
             });
           },
@@ -221,13 +261,13 @@
                 names: [],
                 roles: []
               },
-              readers: {
+              members: {
                 names: [],
                 roles: []
               }
             };
 
-            ["admin", "reader"].forEach(function(key) {
+            ["admin", "member"].forEach(function(key) {
               var names, roles;
 
               try {
@@ -529,13 +569,16 @@
                     location.href = "database.html?" + encodeURIComponent(dbName) +
                       "/" + $.couch.encodeDocId(doc._id) +
                       "/_view/" + encodeURIComponent(data.name);
+                  },
+                  error: function(status, error, reason) {
+                    alert("Error: " + error + "\n\n" + reason);
                   }
                 });
               }
               db.openDoc(docId, {
                 error: function(status, error, reason) {
                   if (status == 404) save(null);
-                  else alert(reason);
+                  else alert("Error: " + error + "\n\n" + reason);
                 },
                 success: function(doc) {
                   save(doc);
@@ -569,8 +612,12 @@
             db.saveDoc(doc, {
               success: function(resp) {
                 page.isDirty = false;
+                page.storedViewCode = viewDef;
                 $("#viewcode button.revert, #viewcode button.save")
                   .attr("disabled", "disabled");
+              },
+              error: function(status, error, reason) {
+                alert("Error: " + error + "\n\n" + reason);
               }
             });
           }
@@ -616,7 +663,7 @@
         $("#documents").find("tbody.content").empty().end().show();
         page.updateDesignDocLink();
 
-        options.success = function(resp) {
+        options.success = function(resp, requestDuration) {
           if (resp.offset === undefined) {
             resp.offset = 0;
           }
@@ -682,10 +729,12 @@
               key = $.futon.formatJSON(row.key, {indent: 0, linesep: ""});
             }
             if (row.id) {
-              $("<td class='key'><a href='document.html?" + encodeURIComponent(db.name) +
-                "/" + $.couch.encodeDocId(row.id) + "'><strong></strong><br>" +
-                "<span class='docid'>ID:&nbsp;" + $.futon.escape(row.id) + "</span></a></td>")
-                .find("strong").text(key).end()
+              key = key.replace(/\\"/, '"');
+              var rowlink = encodeURIComponent(db.name) +
+                "/" + $.couch.encodeDocId(row.id);
+              $("<td class='key'><a href=\"document.html?" + rowlink + "\"><strong>"
+                 + $.futon.escape(key) + "</strong><br>"
+                 + "<span class='docid'>ID:&nbsp;" + $.futon.escape(row.id) + "</span></a></td>")
                 .appendTo(tr);
             } else {
               $("<td class='key'><strong></strong></td>")
@@ -724,6 +773,11 @@
             "Showing " + firstNum + "-" + lastNum + " of " + totalNum +
             " row" + (firstNum != lastNum || totalNum == "unknown" ? "s" : ""));
           $("#documents tbody tr:odd").addClass("odd");
+
+          if (viewName && viewName !== "_all_docs" && viewName !== "_design_docs") {
+            $("#viewrequestduration").show();
+            $("#viewrequestduration .timestamp").text($.toTimeString(requestDuration));
+          }
         }
         options.error = function(status, error, reason) {
           alert("Error: " + error + "\n\n" + reason);
@@ -955,6 +1009,9 @@
 
         if (!page.isNew) {
           db.openDoc(docId, {revs_info: true,
+            error: function(status, error, reason) {
+              alert("Error: " + error + "\n\n" + reason);
+            },
             success: function(doc) {
               var revs = doc._revs_info || [];
               delete doc._revs_info;
@@ -1002,7 +1059,7 @@
           success: function(resp) {
             page.isDirty = false;
             location.href = "?" + encodeURIComponent(dbName) +
-              "/" + $.couch.encodeDocId(page.docId);
+              "/" + $.couch.encodeDocId(resp.id);
           }
         });
       }
@@ -1037,6 +1094,37 @@
         });
       }
 
+      this.copyDocument = function() {
+        if (page.isDirty) {
+          alert("You need to save or revert any changes you have made to the " +
+                "document before you can copy it.");
+            return false;
+        }
+        $.showDialog("dialog/_copy_document.html", {
+          submit: function (data, callback) {
+            if (!data.docid || data.docid.length == 0) {
+              callback({docid: "Please enter a document ID."});
+              return;
+            }
+            db.copyDoc(
+              page.doc._id,
+              {
+                docid: data.docid
+              },
+              {
+                error: function (status, error, reason) {
+                  callback({docid: reason});
+              },
+                success: function (resp) {
+                  location.href = "document.html?"
+                                + encodeURIComponent(dbName)
+                                + "/" + $.couch.encodeDocId(resp.id);
+              }
+            });
+          }
+        });
+      };
+
       window.onbeforeunload = function() {
         if (page.isDirty) {
           return "You've made changes to this document that have not been " +
@@ -1273,8 +1361,7 @@
           return false;
         }).prependTo($("a", li));
       }
-    },
-
+    }
   });
 
   function encodeAttachment(name) {
