@@ -30,7 +30,7 @@ couchTests.changes = function(debug) {
   T(db.save(docFoo).ok);
   T(db.ensureFullCommit().ok);
   T(db.open(docFoo._id)._id == docFoo._id);
-  
+
   req = CouchDB.request("GET", "/test_suite_db/_changes");
   var resp = JSON.parse(req.responseText);
 
@@ -39,7 +39,7 @@ couchTests.changes = function(debug) {
   T(resp.results[0].changes[0].rev == docFoo._rev);
 
   // test with callback
-  
+
   run_on_modified_server(
     [{section: "httpd",
       key: "allow_jsonp",
@@ -78,12 +78,12 @@ couchTests.changes = function(debug) {
     // WebKit (last checked on nightly #47686) does fail on processing
     // the async-request properly while javascript is executed.
 
-    xhr.open("GET", "/test_suite_db/_changes?feed=continuous&timeout=500", true);
+    xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=continuous&timeout=500"), true);
     xhr.send("");
 
     var docBar = {_id:"bar", bar:1};
     db.save(docBar);
-    
+
     var lines, change1, change2;
     waitForSuccess(function() {
       lines = xhr.responseText.split("\n");
@@ -92,16 +92,17 @@ couchTests.changes = function(debug) {
       if (change2.seq != 2) {
           throw "bad seq, try again";
       }
+      return true;
     }, "bar-only");
 
     T(change1.seq == 1);
     T(change1.id == "foo");
-    
+
     T(change2.seq == 2);
     T(change2.id == "bar");
     T(change2.changes[0].rev == docBar._rev);
-    
-    
+
+
     var docBaz = {_id:"baz", baz:1};
     db.save(docBaz);
 
@@ -112,8 +113,9 @@ couchTests.changes = function(debug) {
       if (change3.seq != 3) {
         throw "bad seq, try again";
       }
+      return true;
     });
-    
+
     T(change3.seq == 3);
     T(change3.id == "baz");
     T(change3.changes[0].rev == docBaz._rev);
@@ -122,15 +124,16 @@ couchTests.changes = function(debug) {
     xhr = CouchDB.newXhr();
 
     //verify the hearbeat newlines are sent
-    xhr.open("GET", "/test_suite_db/_changes?feed=continuous&heartbeat=10&timeout=500", true);
+    xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=continuous&heartbeat=10&timeout=500"), true);
     xhr.send("");
-    
+
     var str;
     waitForSuccess(function() {
       str = xhr.responseText;
       if (str.charAt(str.length - 1) != "\n" || str.charAt(str.length - 2) != "\n") {
         throw("keep waiting");
       }
+      return true;
     }, "heartbeat");
 
     T(str.charAt(str.length - 1) == "\n");
@@ -139,27 +142,57 @@ couchTests.changes = function(debug) {
     // otherwise we'll continue to receive heartbeats forever
     xhr.abort();
 
+    // test Server Sent Event (eventsource)
+    if (!!window.EventSource) {
+      var source = new EventSource(
+              "/test_suite_db/_changes?feed=eventsource");
+      var results = [];
+      var sourceListener = function(e) {
+        var data = JSON.parse(e.data);
+        results.push(data);
+      };
+
+      source.addEventListener('message', sourceListener , false);
+
+      waitForSuccess(function() {
+        if (results.length != 3) {
+          throw "bad seq, try again";
+        }
+        return true;
+      });
+
+      source.removeEventListener('message', sourceListener, false);
+
+      T(results[0].seq == 1);
+      T(results[0].id == "foo");
+
+      T(results[1].seq == 2);
+      T(results[1].id == "bar");
+      T(results[1].changes[0].rev == docBar._rev);
+    }
+
     // test longpolling
     xhr = CouchDB.newXhr();
 
-    xhr.open("GET", "/test_suite_db/_changes?feed=longpoll", true);
+    xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=longpoll"), true);
     xhr.send("");
-    
+
     waitForSuccess(function() {
       lines = xhr.responseText.split("\n");
       if (lines[5] != '"last_seq":3}') {
         throw("still waiting");
       }
+      return true;
     }, "last_seq");
-    
+
     xhr = CouchDB.newXhr();
 
-    xhr.open("GET", "/test_suite_db/_changes?feed=longpoll&since=3", true);
+    xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=longpoll&since=3"), true);
     xhr.send("");
 
     var docBarz = {_id:"barz", bar:1};
     db.save(docBarz);
-    
+
     var parse_changes_line = function(line) {
       if (line.charAt(line.length-1) == ",") {
         var linetrimmed = line.substring(0, line.length-1);
@@ -168,14 +201,15 @@ couchTests.changes = function(debug) {
       }
       return JSON.parse(linetrimmed);
     };
-    
+
     waitForSuccess(function() {
       lines = xhr.responseText.split("\n");
       if (lines[3] != '"last_seq":4}') {
         throw("still waiting");
       }
+      return true;
     }, "change_lines");
-    
+
     var change = parse_changes_line(lines[1]);
     T(change.seq == 4);
     T(change.id == "barz");
@@ -183,8 +217,41 @@ couchTests.changes = function(debug) {
     T(lines[3]=='"last_seq":4}');
 
 
+    // test since=now
+    xhr = CouchDB.newXhr();
+
+    xhr.open("GET", "/test_suite_db/_changes?feed=longpoll&since=now", true);
+    xhr.send("");
+
+    var docBarz = {_id:"barzzzz", bar:1};
+    db.save(docBarz);
+
+    var parse_changes_line = function(line) {
+      if (line.charAt(line.length-1) == ",") {
+        var linetrimmed = line.substring(0, line.length-1);
+      } else {
+        var linetrimmed = line;
+      }
+      return JSON.parse(linetrimmed);
+    };
+
+    waitForSuccess(function() {
+      lines = xhr.responseText.split("\n");
+      if (lines[3] != '"last_seq":5}') {
+        throw("still waiting");
+      }
+      return true;
+    }, "change_lines");
+
+    var change = parse_changes_line(lines[1]);
+    T(change.seq == 5);
+    T(change.id == "barzzzz");
+    T(change.changes[0].rev == docBarz._rev);
+    T(lines[3]=='"last_seq":5}');
+
+
   }
-  
+
   // test the filtered changes
   var ddoc = {
     _id : "_design/changes_filter",
@@ -209,7 +276,7 @@ couchTests.changes = function(debug) {
       blah: {
         map : 'function(doc) {' +
               '  if (doc._id == "blah") {' +
-              '    emit(doc._id, null);' +
+              '    emit(null, null);' +
               '  }' +
               '}'
       }
@@ -224,15 +291,15 @@ couchTests.changes = function(debug) {
 
   db.save({"bop" : "foom"});
   db.save({"bop" : false});
-  
+
   var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=changes_filter/bop");
   var resp = JSON.parse(req.responseText);
   T(resp.results.length == 1, "filtered/bop");
-    
+
   req = CouchDB.request("GET", "/test_suite_db/_changes?filter=changes_filter/dynamic&field=woox");
   resp = JSON.parse(req.responseText);
   T(resp.results.length == 0);
-  
+
   req = CouchDB.request("GET", "/test_suite_db/_changes?filter=changes_filter/dynamic&field=bop");
   resp = JSON.parse(req.responseText);
   T(resp.results.length == 1, "changes_filter/dynamic&field=bop");
@@ -241,32 +308,33 @@ couchTests.changes = function(debug) {
     // filter with longpoll
     // longpoll filters full history when run without a since seq
     xhr = CouchDB.newXhr();
-    xhr.open("GET", "/test_suite_db/_changes?feed=longpoll&filter=changes_filter/bop", false);
+    xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=longpoll&filter=changes_filter/bop"), false);
     xhr.send("");
     var resp = JSON.parse(xhr.responseText);
-    T(resp.last_seq == 7);
+    T(resp.last_seq == 8);
     // longpoll waits until a matching change before returning
     xhr = CouchDB.newXhr();
-    xhr.open("GET", "/test_suite_db/_changes?feed=longpoll&since=7&filter=changes_filter/bop", true);
+    xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=longpoll&since=7&filter=changes_filter/bop"), true);
     xhr.send("");
     db.save({"_id":"falsy", "bop" : ""}); // empty string is falsy
     db.save({"_id":"bingo","bop" : "bingo"});
-    
+
     waitForSuccess(function() {
       resp = JSON.parse(xhr.responseText);
+      return true;
     }, "longpoll-since");
-    
-    T(resp.last_seq == 9);
+
+    T(resp.last_seq == 10);
     T(resp.results && resp.results.length > 0 && resp.results[0]["id"] == "bingo", "filter the correct update");
     xhr.abort();
-    
+
     var timeout = 500;
-    var last_seq = 10;
+    var last_seq = 11;
     while (true) {
 
       // filter with continuous
       xhr = CouchDB.newXhr();
-      xhr.open("GET", "/test_suite_db/_changes?feed=continuous&filter=changes_filter/bop&timeout="+timeout, true);
+      xhr.open("GET", CouchDB.proxyUrl("/test_suite_db/_changes?feed=continuous&filter=changes_filter/bop&timeout="+timeout), true);
       xhr.send("");
 
       db.save({"_id":"rusty", "bop" : "plankton"});
@@ -278,6 +346,7 @@ couchTests.changes = function(debug) {
         if (xhr.readyState != 4) {
           throw("still waiting");
         }
+        return true;
       }, "continuous-rusty");
       lines = xhr.responseText.split("\n");
       var good = false;
@@ -302,31 +371,31 @@ couchTests.changes = function(debug) {
   // error conditions
 
   // non-existing design doc
-  var req = CouchDB.request("GET", 
+  var req = CouchDB.request("GET",
     "/test_suite_db/_changes?filter=nothingtosee/bop");
   TEquals(404, req.status, "should return 404 for non existant design doc");
 
-  // non-existing filter 
-  var req = CouchDB.request("GET", 
+  // non-existing filter
+  var req = CouchDB.request("GET",
     "/test_suite_db/_changes?filter=changes_filter/movealong");
   TEquals(404, req.status, "should return 404 for non existant filter fun");
 
   // both
-  var req = CouchDB.request("GET", 
+  var req = CouchDB.request("GET",
     "/test_suite_db/_changes?filter=nothingtosee/movealong");
-  TEquals(404, req.status, 
+  TEquals(404, req.status,
     "should return 404 for non existant design doc and filter fun");
 
   // changes get all_docs style with deleted docs
   var doc = {a:1};
   db.save(doc);
   db.deleteDoc(doc);
-  var req = CouchDB.request("GET", 
+  var req = CouchDB.request("GET",
     "/test_suite_db/_changes?filter=changes_filter/bop&style=all_docs");
   var resp = JSON.parse(req.responseText);
   var expect = (!is_safari && xhr) ? 3: 1;
   TEquals(expect, resp.results.length, "should return matching rows");
- 
+
   // test filter on view function (map)
   //
   T(db.save({"_id":"blah", "bop" : "plankton"}).ok);
@@ -350,7 +419,7 @@ couchTests.changes = function(debug) {
 
       var req = CouchDB.request("GET", "/_session", authOpts);
       var resp = JSON.parse(req.responseText);
-      
+
       T(db.save({"user" : "Noah Slater"}).ok);
       var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=changes_filter/userCtx", authOpts);
       var resp = JSON.parse(req.responseText);
@@ -378,105 +447,107 @@ couchTests.changes = function(debug) {
   resp = JSON.parse(req.responseText);
   T(resp.results.length == 1, "filter=changes_filter/conflicted");
 
+  // diabled because of https://github.com/apache/couchdb/pull/939
   // test with erlang filter function
-  run_on_modified_server([{
-    section: "native_query_servers",
-    key: "erlang",
-    value: "{couch_native_process, start_link, []}"
-  }], function() {
-    var erl_ddoc = {
-      _id: "_design/erlang",
-      language: "erlang",
-      filters: {
-        foo:
-          'fun({Doc}, Req) -> ' +
-          '  case couch_util:get_value(<<"value">>, Doc) of' +
-          '  undefined -> false;' +
-          '  Value -> (Value rem 2) =:= 0;' +
-          '  _ -> false' +
-          '  end ' +
-          'end.'
-      }
-    };
-
-    db.deleteDb();
-    db.createDb();
-    T(db.save(erl_ddoc).ok);
-
-    var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=erlang/foo");
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 0);
-
-    T(db.save({_id: "doc1", value : 1}).ok);
-    T(db.save({_id: "doc2", value : 2}).ok);
-    T(db.save({_id: "doc3", value : 3}).ok);
-    T(db.save({_id: "doc4", value : 4}).ok);
-
-    var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=erlang/foo");
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 2);
-    T(resp.results[0].id === "doc2");
-    T(resp.results[1].id === "doc4");
-
-    // test filtering on docids
-    //
-
-    var options = {
-        headers: {"Content-Type": "application/json"},
-        body: JSON.stringify({"doc_ids": ["something", "anotherthing", "andmore"]})
-    };
-
-    var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 0);
-
-    T(db.save({"_id":"something", "bop" : "plankton"}).ok);
-    var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 1);
-    T(resp.results[0].id === "something");
-
-    T(db.save({"_id":"anotherthing", "bop" : "plankton"}).ok);
-    var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 2);
-    T(resp.results[0].id === "something");
-    T(resp.results[1].id === "anotherthing");
-    
-    var docids = JSON.stringify(["something", "anotherthing", "andmore"]),
-        req = CouchDB.request("GET", "/test_suite_db/_changes?filter=_doc_ids&doc_ids="+docids, options);
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 2);
-    T(resp.results[0].id === "something");
-    T(resp.results[1].id === "anotherthing");
-
-    var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=_design");
-    var resp = JSON.parse(req.responseText);
-    T(resp.results.length === 1);
-    T(resp.results[0].id === "_design/erlang");
-
-
-    if (!is_safari && xhr) {
-        // filter docids with continuous
-        xhr = CouchDB.newXhr();
-        xhr.open("POST", "/test_suite_db/_changes?feed=continuous&timeout=500&since=7&filter=_doc_ids", true);
-        xhr.setRequestHeader("Content-Type", "application/json");
-        
-        xhr.send(options.body);
-        
-        T(db.save({"_id":"andmore", "bop" : "plankton"}).ok);
-
-        waitForSuccess(function() {
-            if (xhr.readyState != 4) {
-              throw("still waiting");
-            }
-        }, "andmore-only");
-
-        var line = JSON.parse(xhr.responseText.split("\n")[0]);
-        T(line.seq == 8);
-        T(line.id == "andmore");
-    }
-  });
+  // run_on_modified_server([{
+  //   section: "native_query_servers",
+  //   key: "erlang",
+  //   value: "{couch_native_process, start_link, []}"
+  // }], function() {
+  //   var erl_ddoc = {
+  //     _id: "_design/erlang",
+  //     language: "erlang",
+  //     filters: {
+  //       foo:
+  //         'fun({Doc}, Req) -> ' +
+  //         '  case couch_util:get_value(<<"value">>, Doc) of' +
+  //         '  undefined -> false;' +
+  //         '  Value -> (Value rem 2) =:= 0;' +
+  //         '  _ -> false' +
+  //         '  end ' +
+  //         'end.'
+  //     }
+  //   };
+  //
+  //   db.deleteDb();
+  //   db.createDb();
+  //   T(db.save(erl_ddoc).ok);
+  //
+  //   var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=erlang/foo");
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 0);
+  //
+  //   T(db.save({_id: "doc1", value : 1}).ok);
+  //   T(db.save({_id: "doc2", value : 2}).ok);
+  //   T(db.save({_id: "doc3", value : 3}).ok);
+  //   T(db.save({_id: "doc4", value : 4}).ok);
+  //
+  //   var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=erlang/foo");
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 2);
+  //   T(resp.results[0].id === "doc2");
+  //   T(resp.results[1].id === "doc4");
+  //
+  //   // test filtering on docids
+  //   //
+  //
+  //   var options = {
+  //       headers: {"Content-Type": "application/json"},
+  //       body: JSON.stringify({"doc_ids": ["something", "anotherthing", "andmore"]})
+  //   };
+  //
+  //   var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 0);
+  //
+  //   T(db.save({"_id":"something", "bop" : "plankton"}).ok);
+  //   var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 1);
+  //   T(resp.results[0].id === "something");
+  //
+  //   T(db.save({"_id":"anotherthing", "bop" : "plankton"}).ok);
+  //   var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 2);
+  //   T(resp.results[0].id === "something");
+  //   T(resp.results[1].id === "anotherthing");
+  //
+  //   var docids = JSON.stringify(["something", "anotherthing", "andmore"]),
+  //       req = CouchDB.request("GET", "/test_suite_db/_changes?filter=_doc_ids&doc_ids="+docids, options);
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 2);
+  //   T(resp.results[0].id === "something");
+  //   T(resp.results[1].id === "anotherthing");
+  //
+  //   var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=_design");
+  //   var resp = JSON.parse(req.responseText);
+  //   T(resp.results.length === 1);
+  //   T(resp.results[0].id === "_design/erlang");
+  //
+  //
+  //   if (!is_safari && xhr) {
+  //       // filter docids with continuous
+  //       xhr = CouchDB.newXhr();
+  //       xhr.open("POST", CouchDB.proxyUrl("/test_suite_db/_changes?feed=continuous&timeout=500&since=7&filter=_doc_ids"), true);
+  //       xhr.setRequestHeader("Content-Type", "application/json");
+  //
+  //       xhr.send(options.body);
+  //
+  //       T(db.save({"_id":"andmore", "bop" : "plankton"}).ok);
+  //
+  //       waitForSuccess(function() {
+  //           if (xhr.readyState != 4) {
+  //             throw("still waiting");
+  //           }
+  //           return true;
+  //       }, "andmore-only");
+  //
+  //       var line = JSON.parse(xhr.responseText.split("\n")[0]);
+  //       T(line.seq == 8);
+  //       T(line.id == "andmore");
+  //   }
+  // });
 
   // COUCHDB-1037 - empty result for ?limit=1&filter=foo/bar in some cases
   T(db.deleteDb());
@@ -550,7 +621,100 @@ couchTests.changes = function(debug) {
   TEquals(1, resp.results.length);
   TEquals(2, resp.results[0].changes.length);
 
+  // COUCHDB-1852
+  T(db.deleteDb());
+  T(db.createDb());
+
+  // create 4 documents... this assumes the update sequnce will start from 0 and get to 4
+  db.save({"bop" : "foom"});
+  db.save({"bop" : "foom"});
+  db.save({"bop" : "foom"});
+  db.save({"bop" : "foom"});
+
+  // simulate an EventSource request with a Last-Event-ID header
+  req = CouchDB.request("GET", "/test_suite_db/_changes?feed=eventsource&timeout=0&since=0",
+        {"headers": {"Accept": "text/event-stream", "Last-Event-ID": "2"}});
+
+  // "parse" the eventsource response and collect only the "id: ..." lines
+  var changes = req.responseText.split('\n')
+     .map(function (el) {
+        return el.split(":").map(function (el) { return el.trim()});
+     })
+     .filter(function (el) { return (el[0] === "id"); })
+
+  // make sure we only got 2 changes, and they are update_seq=3 and update_seq=4
+  T(changes.length === 2);
+  T(changes[0][1] === "3");
+  T(changes[1][1] === "4");
+
+  // COUCHDB-1923
+  T(db.deleteDb());
+  T(db.createDb());
+
+  var attachmentData = "VGhpcyBpcyBhIGJhc2U2NCBlbmNvZGVkIHRleHQ=";
+
+  db.bulkSave(makeDocs(20, 30, {
+    _attachments:{
+      "foo.txt": {
+        content_type:"text/plain",
+        data: attachmentData
+      },
+      "bar.txt": {
+        content_type:"text/plain",
+        data: attachmentData
+      }
+    }
+  }));
+
+  var mapFunction = function(doc) {
+    var count = 0;
+
+    for(var idx in doc._attachments) {
+      count = count + 1;
+    }
+
+    emit(parseInt(doc._id), count);
+  };
+
+  var req = CouchDB.request("GET", "/test_suite_db/_changes?include_docs=true");
+  var resp = JSON.parse(req.responseText);
+
+  T(resp.results.length == 10);
+  T(resp.results[0].doc._attachments['foo.txt'].stub === true);
+  T(resp.results[0].doc._attachments['foo.txt'].data === undefined);
+  T(resp.results[0].doc._attachments['foo.txt'].encoding === undefined);
+  T(resp.results[0].doc._attachments['foo.txt'].encoded_length === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].stub === true);
+  T(resp.results[0].doc._attachments['bar.txt'].data === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].encoding === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].encoded_length === undefined);
+
+  var req = CouchDB.request("GET", "/test_suite_db/_changes?include_docs=true&attachments=true");
+  var resp = JSON.parse(req.responseText);
+
+  T(resp.results.length == 10);
+  T(resp.results[0].doc._attachments['foo.txt'].stub === undefined);
+  T(resp.results[0].doc._attachments['foo.txt'].data === attachmentData);
+  T(resp.results[0].doc._attachments['foo.txt'].encoding === undefined);
+  T(resp.results[0].doc._attachments['foo.txt'].encoded_length === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].stub === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].data == attachmentData);
+  T(resp.results[0].doc._attachments['bar.txt'].encoding === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].encoded_length === undefined);
+
+  var req = CouchDB.request("GET", "/test_suite_db/_changes?include_docs=true&att_encoding_info=true");
+  var resp = JSON.parse(req.responseText);
+
+  T(resp.results.length == 10);
+  T(resp.results[0].doc._attachments['foo.txt'].stub === true);
+  T(resp.results[0].doc._attachments['foo.txt'].data === undefined);
+  T(resp.results[0].doc._attachments['foo.txt'].encoding === "gzip");
+  T(resp.results[0].doc._attachments['foo.txt'].encoded_length === 47);
+  T(resp.results[0].doc._attachments['bar.txt'].stub === true);
+  T(resp.results[0].doc._attachments['bar.txt'].data === undefined);
+  T(resp.results[0].doc._attachments['bar.txt'].encoding === "gzip");
+  T(resp.results[0].doc._attachments['bar.txt'].encoded_length === 47);
+
   // cleanup
   db.deleteDb();
 };
-
